#include <htc.h>
#ifndef __I2C_H
#define __I2C_H

/* PIC18 I2C peripheral library header */

/* SSPCON1 REGISTER */
#ifndef SSPENB
#define   SSPENB        0b00100000           // Enable serial port and configures SCK, SDO, SDI
#endif
#define   SLAVE_7   			0b00000110     /* I2C Slave mode, 7-bit address      */
#define   SLAVE_10  			0b00000111     /* I2C Slave mode, 10-bit address     */
#define   MASTER    			0b00001000     /* I2C Master mode                    */
#define   MASTER_FIRMW			0b00001011
#define   SLAVE_7_STSP_INT 		0b00001110
#define   SLAVE_10_STSP_INT 	0b00001111
/* SSPSTAT REGISTER */
#define   SLEW_OFF  			0b10000000  /* Slew rate disabled for 100kHz mode */
#define   SLEW_ON   			0b00000000  /* Slew rate enabled for 400kHz mode  */


#if defined (_I2C_V2) || defined (_I2C_V3) || defined (_I2C_V5) || defined (_I2C_V6) /* These versions have MSSP1 */

/* ***** I2C1 ***** */

/* StopI2C1
 * Generate bus stop condition
 */
#define StopI2C1()  PEN_x=1
#define StopI2C StopI2C1

/* StartI2C1
 * Generate bus start condition
 */
#define StartI2C1()  SEN_x=1
#define StartI2C StartI2C1 

/* RestartI2C1
 * Generate bus restart condition
 */
#define RestartI2C1()  RSEN_x=1
#define RestartI2C RestartI2C1

/* NotAckI2C1
 * Generate bus Not ACK condition
 */
#define NotAckI2C1()     do { ACKDT_x=1;\
   							  ACKEN_x=1; } while(0)
#define NotAckI2C NotAckI2C1

/* AckI2C1
 * Generate bus ACK condition
 */
#define AckI2C1() do { ACKDT_x=0;\
					   ACKEN_x=1; } while(0)
#define AckI2C AckI2C1

/* Idle I2C1
 * Test if I2C1 module is idle
 */
#define IdleI2C1()    while ((SSP1CON2 & 0x1F) | (R_W_x))
#define IdleI2C IdleI2C11
/* CloseI2C1
 * Disable SPI module
 */
#define CloseI2C1()  SSP1CON1 &=0xDF
#define CloseI2C CloseI2C1
/* OpenI2C1
 * Configure SSP module for use with I2C1 EEPROM or I2C1 bus device.
 */
#define OpenI2C1_BEGIN(sync_mode, slew) SSP1STAT &= 0x3F;\
									    SSP1CON1 = 0x00;\
									    SSP1CON2 = 0x00;\
									    SSP1CON1 |= sync_mode;\
									    SSP1STAT |= slew
#define OpenI2C1_END SSP1CON1 |= SSPENB
#if defined (_I2C_V2) || defined (_I2C_V3)
#define OpenI2C1(sync_mode, slew) do {OpenI2C1_BEGIN(sync_mode, slew);\
									  I2C1_SCL = 1;\
									  I2C1_SDA = 1;\
									  OpenI2C1_END; } while(0)
#endif
#if defined (_I2C_V5) || defined (_I2C_V6)
#define OpenI2C1(sync_mode, slew) do {OpenI2C1_BEGIN(sync_mode, slew);\
	                                  OpenI2C1_END; } while(0)
#endif
#define OpenI2C OpenI2C1

/* DataRdyI2C1
 * Test if SSPBUF register is full
 */
#define DataRdyI2C1()    (BF_x)
#define DataRdyI2C DataRdyI2C1

/* ReadI2C1
 * Read byte from SSP1BUF register
 */
unsigned char ReadI2C1( void );
#define ReadI2C ReadI2C1

/* GetcI2C1
 * Read byte from SSP1BUF register
 */
#define  getcI2C1  ReadI2C1
#define  getcI2C getcI2C1

/* WriteI2C1
 * Write byte to SSP1BUF register
 */
unsigned char WriteI2C1(  unsigned char data_out );
#define WriteI2C WriteI2C1

/* putcI2C1
 * Write byte to SSP1BUF register
 */
#define  putcI2C1  WriteI2C1
#define  putcI2C putcI2C1

/* getsI2C1
 * Read in a string from I2C1 module
 */
unsigned char getsI2C1(  unsigned char *rdptr,  unsigned char length );
#define getsI2C getsI2C1

/* putsI2C1
 * Write a string to I2C1 module
 */
unsigned char putsI2C1(  unsigned char *wrptr );
#define putsI2C putsI2C1

/* EEAckPolling1
 */
unsigned char EEAckPolling1(  unsigned char control );
#define EEAckPolling EEAckPolling1

/* EEByteWrite1
 */
unsigned char EEByteWrite1(  unsigned char control,
                            unsigned char address,
                            unsigned char data );
#define EEByteWrite EEByteWrite1

/* EECurrentAddRead1
 */
unsigned int  EECurrentAddRead1(  unsigned char control );
#define EECurrentAddRead EECurrentAddRead1

/* EEPageWrite1
 */
unsigned char EEPageWrite1(  unsigned char control,
                            unsigned char address,
                            unsigned char *wrptr );
#define EEPageWrite EEPageWrite1

/* EERandomRead1
 */
unsigned int  EERandomRead1(  unsigned char control,  unsigned char address );
#define EERandomRead EERandomRead1

/* EESequentialRead1
 */
unsigned char EESequentialRead1(  unsigned char control,
                                 unsigned char address,
                                 unsigned char *rdptr,
                                 unsigned char length );
#define EESequentialRead EESequentialRead1
#endif

#if defined (_I2C_V3) || defined (_I2C_V6) /*This version has MSSP2*/
/* ***** I2C2 ***** */

/* StopI2C2
 * Generate bus stop condition
 */
#define StopI2C2()  PEN_x=1

/* StartI2C2
 * Generate bus start condition
 */
#define StartI2C2()  SEN_x=1

/* RestartI2C2
 * Generate bus restart condition
 */
#define RestartI2C2() RSEN_x=1

/* NotAckI2C2
 * Generate bus Not ACK condition
 */
#define NotAckI2C2() do {ACKDT_x=1;\
   						 ACKEN_x=1; } while(0)

/* AckI2C2
 * Generate bus ACK condition
 */
#define AckI2C2() do { ACKDT_x=0;\
   					   ACKEN_x=1; } while(0)

/* Idle I2C2
 * Test if I2C2 module is idle
 */
#define IdleI2C2()    while ((SSP2CON2 & 0x1F) | (R_W_x))

/* CloseI2C2
 * Disable SPI module
 */
#define CloseI2C2()  SSP2CON1 &=0xDF

/* OpenI2C2
 * Configure SSP module for use with I2C2 EEPROM or I2C2 bus device.
 */
#define OpenI2C2_BEGIN(sync_mode, slew) SSP2STAT &= 0x3F;\
										SSP2CON1 = 0x00;\
                                        SSP2CON2 = 0x00;\
                                        SSP2CON1 |= sync_mode;\
                                        SSP2STAT |= slew
#define OpenI2C2_END SSP2CON1 |= SSPENB
#if defined (_I2C_V3)
#define OpenI2C2(sync_mode, slew) do {OpenI2C2_BEGIN(sync_mode, slew);\
	                                  I2C2_SCL = 1;\
	                                  I2C2_SDA = 1;\
	                                  OpenI2C2_END; } while(0)
#endif
#if defined (_I2C_V6)
#define OpenI2C2(sync_mode, slew) do {OpenI2C2_BEGIN(sync_mode, slew);\
	                                  OpenI2C2_END; } while(0)
#endif




/* DataRdyI2C2
 * Test if SSPBUF register is full
 */
#define DataRdyI2C2()    (BF_x)

/* ReadI2C2
 * Read byte from SSP2BUF register
 */
unsigned char ReadI2C2( void );

/* GetcI2C2
 * Read byte from SSP2BUF register
 */
#define  getcI2C2  ReadI2C2

/* WriteI2C2
 * Write byte to SSP2BUF register
 */
unsigned char WriteI2C2(  unsigned char data_out );

/* putcI2C2
 * Write byte to SSP2BUF register
 */
#define  putcI2C2  WriteI2C2

/* getsI2C2
 * Read in a string from I2C2 module
 */
unsigned char getsI2C2(  unsigned char *rdptr,  unsigned char length );

/* putsI2C2
 * Write a string to I2C2 module
 */
signed char putsI2C2(  unsigned char *wrptr );

/* EEAckPolling2
 */
unsigned char EEAckPolling2(  unsigned char control );

/* EEByteWrite2
 */
unsigned char EEByteWrite2(  unsigned char control,
                            unsigned char address,
                            unsigned char data );

/* EECurrentAddRead2
 */
unsigned int  EECurrentAddRead2(  unsigned char control );

/* EEPageWrite2
 */
unsigned char EEPageWrite2(  unsigned char control,
                            unsigned char address,
                            unsigned char *wrptr );

/* EERandomRead2
 */
unsigned int  EERandomRead2(  unsigned char control,  unsigned char address );

/* EESequentialRead2
 */
unsigned char EESequentialRead2(  unsigned char control,
                                 unsigned char address,
                                 unsigned char *rdptr,
                                 unsigned char length );
#endif

#if defined (_I2C_V1) || defined (_I2C_V4)    /* Parts with only one I2C, MSSP */

/* GetcI2C
 * Read byte from SSPBUF register
 */
#define  getcI2C  ReadI2C

/* putsI2C
 * Write a string to I2C module
 */
signed char putsI2C(  unsigned char *wrptr );

/* ReadI2C
 * Read byte from SSPBUF register
 */
unsigned char ReadI2C( void );

#endif

#if defined (_I2C_V1) 

/* CloseI2C
 * Disable SPI module
 */
#define CloseI2C()  SSPCON1 &=0xDF

/* OpenI2C
 * Open SPI module
 */

#define OpenI2C_BEGIN(sync_mode, slew) SSPSTAT &= 0x3F;\
									   SSPCON1 = 0x00;\
                                       SSPCON2 = 0x00;\
                                       SSPCON1 |= sync_mode;\
                                       SSPSTAT |= slew
#define OpenI2C_END SSPCON1 |= SSPENB
#define OpenI2C(sync_mode, slew) do {OpenI2C_BEGIN(sync_mode, slew);\
	                                 I2C_SCL = 1;\
	                                 I2C_SDA = 1;\
	                                 OpenI2C_END; } while(0)
#if defined (_I2C_V2) || defined (_I2C_V3)
#define OpenI2C(sync_mode, slew) do {OpenI2C_BEGIN(sync_mode, slew);\
	                                 I2C_SCL = 1;\
	                                 I2C_SDA = 1;\
	                                 OpenI2C_END; } while(0)
#endif
#if defined (_I2C_V5) || defined (_I2C_V6)
#define OpenI2C(sync_mode, slew) do {OpenI2C_BEGIN(sync_mode, slew);\
	                                 OpenI2C_END; } while(0)
#endif


/* Idle I2C
 * Test if I2C module is idle
 */
#define IdleI2C()    while ((SSPCON2 & 0x1F) | (R_W_x))

/* StopI2C
 * Generate bus stop condition
 */
#define StopI2C()  PEN=1

/* StartI2C
 * Generate bus start condition
 */
#define StartI2C() SEN_x=1

/* RestartI2C
 * Generate bus restart condition
 */
#define RestartI2C() RSEN_x=1

/* NotAckI2C
 * Generate bus Not ACK condition
 */
#define NotAckI2C() do { ACKDT_x=1;\
   						 ACKEN_x=1; } while(0)

/* AckI2C
 * Generate bus ACK condition
 */
#define AckI2C() do { ACKDT_x=0;\
   					  ACKEN_x=1; } while(0)

/* DataRdyI2C
 * Test if SSPBUF register is full
 */
#define DataRdyI2C()    (BF_x)

/* putcI2C
 * Write byte to SSPBUF register
 */
#define  putcI2C  WriteI2C

/* WriteI2C
 * Write byte to SSPBUF register
 */
unsigned char WriteI2C(  unsigned char data_out );

/* getsI2C
 * Read in a string from I2C module
 */
unsigned char getsI2C(  unsigned char *rdptr,  unsigned char length );

/* EEAckPolling
 */
unsigned char EEAckPolling(  unsigned char control );

/* EEByteWrite
 */
unsigned char EEByteWrite(  unsigned char control,
                            unsigned char address,
                            unsigned char data );

/* EECurrentAddRead
 */
unsigned int  EECurrentAddRead(  unsigned char control );

/* EEPageWrite
 */
unsigned char EEPageWrite(  unsigned char control,
                            unsigned char address,
                            unsigned char *wrptr );

/* EERandomRead
 */
unsigned int  EERandomRead(  unsigned char control,  unsigned char address );

/* EESequentialRead
 */
unsigned char EESequentialRead(  unsigned char control,
                                 unsigned char address,
                                 unsigned char *rdptr,
                                 unsigned char length );
#endif

#if defined (_I2C_V4)
#define CloseI2C()  SSPCON &=0xDF
#define IdleI2C()    while ((SSPCON & 0x1F) | (R_W_x)) 

/* OpenI2C
 *  * Open SPI module
 *   */

#define OpenI2C_BEGIN(sync_mode, slew) SSPSTAT &= 0x3F;\
									   SSPCON = 0x00;\
                                       SSPCON = 0x00;\
                                       SSPCON |= sync_mode;\
                                       SSPSTAT |= slew
#define OpenI2C_END SSPCON |= SSPENB
#if defined (_I2C_V2) || defined (_I2C_V3)
#define OpenI2C(sync_mode, slew) do {OpenI2C_BEGIN(sync_mode, slew);\
	                                 I2C_SCL = 1;\
	                                 I2C_SDA = 1;\
	                                 OpenI2C_END; } while(0)
#endif
#if defined (_I2C_V5) || defined (_I2C_V6)
#define OpenI2C(sync_mode, slew) do {OpenI2C_BEGIN(sync_mode, slew);\
	                                 OpenI2C_END; } while(0)
#endif
#endif

#if defined (_18C242) ||defined (_18C252) ||defined (_18C442) ||\
defined (_18C452) ||defined (_18F242) ||defined (_18F252) ||\
defined (_18F442) ||defined (_18F452) ||defined (_18F248) ||\
defined (_18F258) ||defined (_18F448) ||defined (_18F458) ||\
defined (_18F2439) ||defined (_18F2539) ||defined (_18F4439) ||\
defined (_18F4539) ||defined (_18C601) ||defined (_18C801) ||\
defined (_18C658) ||defined (_18C858) ||defined (_18F6620) ||\
defined (_18F6720) ||defined (_18F8620) ||defined (_18F8720) ||\
defined (_18F6520) ||defined (_18F8520) ||defined (_18F2220) ||\
defined (_18F2320) ||defined (_18F4220) ||defined (_18F4320) ||\
defined (_18F2420) ||defined (_18F2520) ||defined (_18F4420) ||\
defined (_18F4520) ||defined (_18F2423) ||defined (_18F2523) ||\
defined (_18F4423) ||defined (_18F4523) ||defined (_18F2480) ||\
defined (_18F2580) ||defined (_18F4480) ||defined (_18F4580) ||\
defined (_18F2410) ||defined (_18F2510) ||defined (_18F2515) ||\
defined (_18F2610) ||defined (_18F4410) ||defined (_18F4510) ||\
defined (_18F4515) ||defined (_18F4610) ||defined (_18F2525) ||\
defined (_18F2620) ||defined (_18F4525) ||defined (_18F4620) ||\
defined (_18F2585) ||defined (_18F2680) ||defined (_18F4585) ||\
defined (_18F4680) ||defined (_18F2682) ||defined (_18F2685) ||\
defined (_18F4682) ||defined (_18F4685) ||defined (_18F2221) ||\
defined (_18F2321) ||defined (_18F4221) ||defined (_18F4321) ||\
defined (_18F6310) ||defined (_18F6410) ||defined (_18F8310) ||\
defined (_18F8410) ||defined (_18F6390) ||defined (_18F6490) ||\
defined (_18F8390) ||defined (_18F8490) ||defined (_18F6585) ||\
defined (_18F6680) ||defined (_18F8585) ||defined (_18F8680) ||\
defined (_18F6525) ||defined (_18F6621) ||defined (_18F8525) ||\
defined (_18F8621) ||defined (_18F63J11) ||defined (_18F64J11) ||\
defined (_18F65J11) ||defined (_18F83J11) ||defined (_18F84J11) ||\
defined (_18F85J11) ||defined (_18F63J90) ||defined (_18F64J90) ||\
defined (_18F65J90) ||defined (_18F83J90) ||defined (_18F84J90) ||\
defined (_18F85J90) ||defined (_18F23K20) ||defined (_18F24K20) ||\
defined (_18F25K20) ||defined (_18F26K20) ||defined (_18F43K20) ||\
defined (_18F44K20) ||defined (_18F45K20) ||defined (_18F46K20 )||\
defined (_18F6493) || defined (_18F8493) ||\
defined (_18F6393) || defined (_18F8393) 
#define I2C_SCL	TRISC3
#define I2C_SDA	TRISC4
#endif

#if defined (_18F2331) ||defined (_18F2431) ||defined (_18F4331) || defined (_18F4431)
#define I2C_SCL	TRISC5
#define I2C_SDA	TRISC4
#endif

#if defined (_18F2455) ||defined (_18F2550) ||defined (_18F4455) ||\
	defined (_18F4550) ||defined (_18F2458) ||defined (_18F2553) ||\
	defined (_18F4458) ||defined (_18F4553)
#define I2C_SCL	TRISB1
#define I2C_SDA	TRISB0
#endif

#if defined (_18F13K50) || defined (_18F14K50) ||\
    defined (_18LF13K50) || defined (_18LF14K50) ||\
    defined (_18F13K20) || defined (_18F14K20) ||\
    defined (_18F13K22) || defined (_18F14K22) ||\
    defined (_18LF13K22) || defined (_18LF14K22)
#define I2C_SCL	TRISB6
#define I2C_SDA	TRISB4

#endif

#if defined (_18F6527) ||defined (_18F6622) ||defined (_18F6627) ||\
defined (_18F6722) ||defined (_18F8527) ||defined (_18F8622) ||\
defined (_18F8627) ||defined (_18F8722) ||defined (_18F24J10) ||\
defined (_18F25J10) ||defined (_18F44J10) ||defined (_18F45J10) ||\
defined (_18F65J10) ||defined (_18F65J15) ||defined (_18F66J10) ||\
defined (_18F66J15) ||defined (_18F67J10) ||defined (_18F85J10) ||\
defined (_18F85J15) ||defined (_18F86J10) ||defined (_18F86J15) ||\
defined (_18F87J10) ||defined (_18F66J60) ||defined (_18F66J65) ||\
defined (_18F67J60) ||defined (_18F86J60) ||defined (_18F86J65) ||\
defined (_18F87J60) ||defined (_18F96J60) ||defined (_18F96J65) ||\
defined (_18F97J60) ||defined (_18F66J11) ||defined (_18F66J16) ||\
defined (_18F67J11) ||defined (_18F86J11) ||defined (_18F86J16) ||\
defined (_18F87J11) ||defined (_18F65J50) ||defined (_18F66J50) ||\
defined (_18F66J55) ||defined (_18F67J50) ||defined (_18F85J50) ||\
defined (_18F86J50) ||defined (_18F86J55) ||defined (_18F87J50) ||\
defined (_18F6628)  || defined (_18F6723) || defined (_18F8628) ||\
defined (_18F8723)
#define I2C1_SCL	RC3
#define I2C1_SDA	RC4
#endif

#if defined (_18F44J10) || defined (_18F45J10)
#define I2C2_SCL	RD0
#define I2C2_SDA	RD1
#endif

#if defined (_18F6527) ||defined (_18F6622) ||defined (_18F6627) ||\
defined (_18F6722) ||defined (_18F8527) ||defined (_18F8622) ||\
defined (_18F8627) ||defined (_18F8722) ||defined (_18F65J10) ||\
defined (_18F65J15) ||defined (_18F66J10) ||defined (_18F66J15) ||\
defined (_18F67J10) ||defined (_18F85J10) ||defined (_18F85J15) ||\
defined (_18F86J10) ||defined (_18F86J15) ||defined (_18F87J10) ||\
defined (_18F96J60) ||defined (_18F96J65) ||defined (_18F97J60) ||\
defined (_18F66J11) ||defined (_18F66J16) ||defined (_18F67J11) ||\
defined (_18F86J11) ||defined (_18F86J16) ||defined (_18F87J11) ||\
defined (_18F65J50) ||defined (_18F66J50) ||defined (_18F66J55) ||\
defined (_18F67J50) ||defined (_18F85J50) ||defined (_18F86J50) ||\
defined (_18F86J55) ||defined (_18F87J50) ||defined (_18F6628)  ||\
defined (_18F6723)  || defined (_18F8628) ||defined (_18F8723)
#define I2C2_SCL	RD6
#define I2C2_SDA	.RD5
#endif

#endif

